# This code is copyrighted by iuveno AG in July, 2001.
#
# It is Open Source. See the LICENSE.txt for more details!
#
# written by Stephan Richter
# Last updated: July 17, 2001
from AccessControl.User import User, UserFolder, reqattr
from Globals import DTMLFile, MessageDialog
import string

def manage_addSlaveUserFolder(self, dtself=None, REQUEST=None, **ignored):
    '''manage_addSlaveUserFolder(self, dtself=None, REQUEST=None, **ignored) -->
       add a Slave User Folder to the directory. Note that it behaves like a
       true User Folder not allowing an further User Folders to be added.'''

    # create a new slave user folder
    uf = SlaveUserFolder()

    # See whether we can still add another user folder in this folderish object
    self = self.this()
    try:
        self._setObject('acl_users', uf)
    except:
        return MessageDialog(title  ='Item Exists',
                             message='This object already contains a User Folder',
                             action ='%s/manage_main' % REQUEST['URL1'])

    # register the user folder
    self.__allow_groups__ = uf
    
    # return to the main screen of the folderish object
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')


class SlaveUserFolder(UserFolder):
    '''SlaveUserFolder class - A User Folder implementation that gets the authentication
       information from another User Folder.'''

    meta_type = 'Slave User Folder'
    title = 'Slave User Folder'

    # This is necessary, so that our User Folder Find Method does not pick a Slave
    # User Folder accidently.
    isASlaveUserFolder = 1

    _add_SlaveUser  = DTMLFile('dtml/addSlaveUser', globals())
    _edit_SlaveUser = DTMLFile('dtml/editSlaveUser', globals())


    # Okay, I had to re-implement this method, so that we do not loose the
    # context; __of__ takes care of that for us
    def getUser(self, name):
        """Return the named user object or None"""
        try:
            return self.data.get(name).__of__(self)
        except AttributeError:
            None


    def _addUser(self, name, roles, REQUEST=None):
        '''_addUser(self, name, roles, REQUEST=None) --> add a slave user to the slave user folder;
           return HTML'''
        if not name:
            return MessageDialog(
                   title  ='Illegal value', 
                   message='A username must be specified',
                   action ='manage_main')

        if self.getUser(name):
            return MessageDialog(
                   title  ='Illegal value', 
                   message='A user with the specified name already exists.',
                   action ='manage_main')

        if not roles: roles=[]

        self._doAddUser(name, roles)        
        if REQUEST:
            return self._mainUser(self, REQUEST)


    def _changeUser(self, name, roles, REQUEST=None):
        '''_changeUser(self, name, roles, REQUEST=None) --> change the user; return HTML'''
        if not name:
            return MessageDialog(
                   title  ='Illegal value', 
                   message='A username must be specified',
                   action ='manage_main')

        if not self.getUser(name):
            return MessageDialog(
                   title  ='Illegal value', 
                   message='A user with the specified name does not exist.',
                   action ='manage_main')

        if not roles:
            roles=[]

        self._doChangeUser(name, roles)
        if REQUEST:
            return self._mainUser(self, REQUEST)


    def manage_users(self, submit=None, REQUEST=None, RESPONSE=None):
        '''manage_users(self, submit=None, REQUEST=None, RESPONSE=None) --> manage user interface;
           return HTML'''

        if submit == 'Add...':
            return self._add_SlaveUser(self, REQUEST)

        if submit == 'Edit':
            try:
                user = self.getUser(reqattr(REQUEST, 'name'))
            except:
                return MessageDialog(
                    title  ='Illegal value',
                    message='The specified user does not exist',
                    action ='manage_main')
            return self._edit_SlaveUser(self, REQUEST, user=user)

        if submit == 'Add':
            name    = reqattr(REQUEST, 'name')
            roles   = reqattr(REQUEST, 'roles')
            return self._addUser(name, roles, REQUEST)

        if submit == 'Change':
            name     = reqattr(REQUEST, 'name')
            roles    = reqattr(REQUEST, 'roles')
            return self._changeUser(name, roles, REQUEST)

        if submit == 'Delete':
            names = reqattr(REQUEST, 'names')
            return self._delUsers(names, REQUEST)

        return self._mainUser(self, REQUEST)


    def _doAddUser(self, name, roles):
        '''_doAddUser(self, name, roles) --> Create a new slave user; return None'''
        self.data[name] = SlaveUser(name, roles, string.join(self.getPhysicalPath(), '/'))


    def _doChangeUser(self, name, roles):
        '''_doChangeUser(self, name, roles) --> Change a slave user; return None'''
        user       = self.data[name]
        user.roles = roles


    def getMasterUsers(self):
        '''getMasterUsers(self) --> Get a list of all master users (with all the info;
           return List of User'''
        userFolder = findUserFolder(self)
        return userFolder.getUsers()


    def getFreeMasterUsers(self):
        '''getFreeMasterUsers(self) --> Get master users that do not have a presence in
           this folder already; return list of User'''
        masters = self.getMasterUsers()
        localNames = map(lambda user: user.name, self.getUsers())
        freeMasters = []
        for user in masters:
            if user.name not in localNames:
                freeMasters.append(user)

        return freeMasters
    

    def getMasterUserFolderPath(self):
        '''getMasterUserFolderPath(self) --> Get the path of the master user folder;
           return string'''
        userFolder = findUserFolder(self)
        
        object = userFolder
        path = ''
        while object:
            if hasattr(object, 'getId') and object.getId():
                path = '/%s' %object.getId() + path

            if hasattr(object, 'aq_parent') and object.aq_parent:
                object = object.aq_parent
            else:
                object = None

        return path



class SlaveUser(User):
    '''SlaveUser class - A user that has all its relevant information stored at another place
                         and just defines some local roles here.'''

    def __init__(self, name, roles, folderPath=None):
        '''__init__(self, name, roles, folderPath=None) --> initialize the object; return None'''
        self.name       = name
        self.roles      = roles
        self.folderPath = folderPath


    def getUserName(self):
        '''getUserName(self) --> Return the username of the user; return string'''
        return self.name


    def _getPassword(self):
        '''_getPassword(self) --> Return the password of the user; return string'''
        # Since the password is not stored locally, we need to get it first
        try:
            return self.getMasterUser().__
        except:
            raise 'Authentication Error', 'The User was not found!'
        

    def getRoles(self):
        '''getRoles(self) --> Return the list of roles assigned to the user;
           return list of string'''
        return tuple(self.roles)


    def getDomains(self):
        '''getDomains(self) --> Return the list of domain restrictions for a user;
           return list of string'''
        # The domains are also stored remotely in the master user folder
        return tuple(self.getMasterUser().domains)


    def getMasterUser(self):
        '''getMasterUser(self) --> get the real user that corresponds to this one;
           return User'''
        userFolder = findUserFolder(self)
        return userFolder.getUser(self.name)



def findUserFolder(folder):
    '''findUserFolder(folder) --> Find next User Folder; return UserFolder'''
    while (folder is not None):
        if hasattr(folder, 'objectValues'):
            for object in folder.objectValues():
                if ( ( hasattr(object, 'isAUserFolder')      and object.isAUserFolder) and not
                     ( hasattr(object, 'isASlaveUserFolder') and object.isASlaveUserFolder) ):
                    return object

        if hasattr(folder, 'aq_parent'):
            folder = folder.aq_parent
        else:
            folder = None

    return None
